package com.qmth.distributed.print.business.service.impl;

import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.itextpdf.text.DocumentException;
import com.itextpdf.text.pdf.PdfReader;
import com.qmth.boot.api.exception.ApiException;
import com.qmth.distributed.print.business.bean.dto.PdfDto;
import com.qmth.distributed.print.business.bean.dto.PdfPackageDto;
import com.qmth.distributed.print.business.bean.dto.PdfSignDto;
import com.qmth.distributed.print.business.bean.params.ExamTaskStudentObjectParam;
import com.qmth.distributed.print.business.bean.params.SerialNumberParams;
import com.qmth.distributed.print.business.entity.*;
import com.qmth.distributed.print.business.enums.ExamDetailStatusEnum;
import com.qmth.distributed.print.business.enums.PrintPlanStatusEnum;
import com.qmth.distributed.print.business.enums.StudentClazzEnum;
import com.qmth.distributed.print.business.service.*;
import com.qmth.distributed.print.business.util.CreatePrintPdfUtil;
import com.qmth.distributed.print.business.util.HtmlToPdfUtil;
import com.qmth.distributed.print.business.util.PdfUtil;
import com.qmth.teachcloud.common.bean.dto.MqDto;
import com.qmth.teachcloud.common.bean.params.ArraysParams;
import com.qmth.teachcloud.common.bean.result.BasicStudentResult;
import com.qmth.teachcloud.common.config.DictionaryConfig;
import com.qmth.teachcloud.common.contant.SpringContextHolder;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.*;
import com.qmth.teachcloud.common.enums.*;
import com.qmth.teachcloud.common.mapper.BasicCourseMapper;
import com.qmth.teachcloud.common.service.*;
import com.qmth.teachcloud.common.util.*;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.io.FilenameUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.FileCopyUtils;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.*;
import java.math.BigDecimal;
import java.nio.charset.StandardCharsets;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;
import java.util.stream.Collectors;

/**
 * @Description: 公共服务service impl
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2021/3/26
 */
@Service
public class PrintCommonServiceImpl implements PrintCommonService {
    private final static Logger log = LoggerFactory.getLogger(PrintCommonServiceImpl.class);

    Lock lock = new ReentrantLock();

    @Resource
    private DictionaryConfig dictionaryConfig;

    @Resource
    BasicAttachmentService basicAttachmentService;

    @Resource
    @Lazy
    private ExamTaskService examTaskService;

    @Resource
    private BasicCourseMapper basicCourseMapper;

    @Resource
    BasicCourseService basicCourseService;

    @Resource
    ExamDetailCourseService examDetailCourseService;

    @Resource
    SysOrgService sysOrgService;

    @Resource
    private SysUserService sysUserService;

    @Resource
    TBTaskService tbTaskService;

    @Resource
    TBSyncTaskService tbSyncTaskService;

    @Resource
    @Lazy
    ExamDetailService examDetailService;

    @Resource
    private ConvertUtil convertUtil;

    @Resource
    @Lazy
    ExamTaskDetailService examTaskDetailService;

    @Resource
    ExamPrintPlanService examPrintPlanService;

    @Resource
    TFFlowApproveService tfFlowApproveService;

    @Resource
    private FileStoreUtil fileStoreUtil;

    @Resource
    TeachcloudCommonService teachcloudCommonService;

    @Resource
    ExamStudentService examStudentService;

    @Resource
    RedisUtil redisUtil;

    @Resource
    BasicCardRuleService basicCardRuleService;

    @Resource
    BasicExamService basicExamService;

    @Resource
    GradeBatchPaperService gradeBatchPaperService;

    @Resource
    BasicStudentService basicStudentService;

    @Resource
    TeachClazzService teachClazzService;

    @Resource
    BasicClazzService basicClazzService;

    @Resource
    BasicExamRuleService basicExamRuleService;

    @Resource
    CreatePrintPdfUtil createPrintPdfUtil;

    @Resource
    CommonCacheService commonCacheService;

    @Resource
    BasicRoleDataPermissionService basicRoleDataPermissionService;

    /**
     * 保存附件
     *
     * @param classifyEnum
     * @param examDetail
     * @param basicAttachment
     * @param pdfList
     * @param printCount
     * @param sequence
     * @param fileTempList
     * @return
     * @throws IOException
     */
    @Override
    @Transactional
    public BasicAttachment saveAttachmentPdf(ClassifyEnum classifyEnum, ExamDetail examDetail, BasicAttachment basicAttachment, List<PdfDto> pdfList, Integer printCount, Integer sequence, List<File> fileTempList) throws IOException, DocumentException {
        try {
            boolean oss = dictionaryConfig.sysDomain().isOss();
            JSONObject jsonObject = JSONObject.parseObject(basicAttachment.getPath());
            String type = basicAttachment.getType(), filePath = null, url = null, ossStr = null;
            File htmlFile = null;

            StringJoiner pdfStringJoiner = new StringJoiner("");
            if (Objects.nonNull(type) && Objects.equals(type, SystemConstant.HTML_PREFIX)) {
                filePath = (String) jsonObject.get(SystemConstant.PATH);
                UploadFileEnum uploadType = Enum.valueOf(UploadFileEnum.class, (String) jsonObject.get(SystemConstant.UPLOAD_TYPE));
                if (oss || (!oss && dictionaryConfig.fssPrivateDomain().getConfig().startsWith(SystemConstant.START_PARENT))) {
                    ossStr = oss ? SystemConstant.OSS : SystemConstant.LOCAL;
                    File htmlFileTemp = SystemConstant.getFileTempVar(SystemConstant.HTML_PREFIX);
                    fileTempList.add(htmlFileTemp);
                    htmlFile = fileStoreUtil.ossDownload(filePath, htmlFileTemp, uploadType.getFssType());
                } else {
                    ossStr = SystemConstant.LOCAL;
                    if (Objects.nonNull(dictionaryConfig.fssPublicDomain()) && !StringUtils.isBlank(dictionaryConfig.fssPublicDomain().getConfig()) && filePath.contains(dictionaryConfig.fssPublicDomain().getConfig())) {
                        htmlFile = new File(filePath);
                    } else {
                        File htmlFileTemp = SystemConstant.getFileTempVar(SystemConstant.HTML_PREFIX);
                        htmlFile = fileStoreUtil.ossDownload(filePath, htmlFileTemp, uploadType.getFssType());
                    }
                    if (Objects.nonNull(dictionaryConfig.fssPrivateDomain()) && !StringUtils.isBlank(dictionaryConfig.fssPrivateDomain().getConfig()) && !dictionaryConfig.fssPrivateDomain().getConfig().startsWith(SystemConstant.START_PARENT)) {
                        pdfStringJoiner.add(dictionaryConfig.fssPrivateDomain().getConfig()).add(File.separator);
                    }
                }
            }

            pdfStringJoiner = SystemConstant.getDirName(pdfStringJoiner, UploadFileEnum.PDF, true);
            pdfStringJoiner.add(SystemConstant.getNanoId()).add(SystemConstant.PDF_PREFIX);
            String pdfDirName = pdfStringJoiner.toString();
            File pdfFileTemp = SystemConstant.getFileTempVar(SystemConstant.PDF_PREFIX);
            fileTempList.add(pdfFileTemp);
            HtmlToPdfUtil.convert(htmlFile.getPath(), pdfFileTemp.getPath(), PageSizeEnum.A4);

//            File pdfFile = asposePdfUtil.documentToPdf(htmlFile.getPath(), destUrl, PaperSize.A4);
            String htmlFileMd5 = DigestUtils.md5Hex(new FileInputStream(htmlFile));
            String pdfFileMd5 = DigestUtils.md5Hex(new FileInputStream(pdfFileTemp));
            if (oss || (!oss && dictionaryConfig.fssPrivateDomain().getConfig().startsWith(SystemConstant.START_PARENT))) {
                fileStoreUtil.ossUpload(pdfDirName, pdfFileTemp, pdfFileMd5, fileStoreUtil.getUploadEnumByPath(pdfDirName).getFssType());
            } else {
                fileStoreUtil.localUpload(pdfDirName, new FileInputStream(pdfFileTemp), pdfFileMd5, LocalCatalogEnum.LOCAL_PDF);
            }

            PdfDto pdfDto = PdfUtil.addPdfPage(pdfFileTemp);
            JSONObject attachmentPath = JSONObject.parseObject(examDetail.getAttachmentPath());
            attachmentPath = Objects.isNull(attachmentPath) ? new JSONObject() : attachmentPath;
            JSONArray jsonArray = (JSONArray) attachmentPath.get(SystemConstant.PATH);
            jsonArray = Objects.isNull(jsonArray) ? new JSONArray() : jsonArray;
            JSONObject object = new JSONObject();
            object.put("printType", classifyEnum.name());
            object.put(SystemConstant.HTML_PATH, filePath);
            object.put("htmlMd5", htmlFileMd5);
            object.put("pdfMd5", pdfFileMd5);
            object.put(SystemConstant.PDF_PATH, pdfDirName);
            object.put(SystemConstant.TYPE, ossStr);
            object.put(SystemConstant.UPLOAD_TYPE, UploadFileEnum.PDF);
            jsonArray.add(object);
            attachmentPath.put(SystemConstant.PATH, jsonArray);
            examDetail.setAttachmentPath(attachmentPath.toJSONString());
            basicAttachment.setPages(pdfDto.getActualPageCount());
            for (int i = 0; i < printCount; i++) {
                pdfList.add(new PdfDto(pdfFileTemp.getPath(), PageSizeEnum.A4, pdfDto.getPageCount(), sequence));
            }
        } catch (Exception e) {
            log.error(SystemConstant.LOG_ERROR, e);
            if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        }
        return basicAttachment;
    }

    /**
     * 保存附件
     *
     * @param examDetail
     * @param pdfList
     * @param printCount
     * @param fileTempList
     */
    @Override
    @Transactional
    public void saveAttachmentSignPdf(PdfSignDto pdfFillDto, ExamDetail examDetail, List<PdfDto> pdfList, Integer printCount, List<File> fileTempList, Boolean diallel) {
        try {
            boolean oss = dictionaryConfig.sysDomain().isOss();
            StringJoiner pdfStringJoiner = new StringJoiner("");
            String ossStr = null;
            if (!oss && Objects.nonNull(dictionaryConfig.fssPrivateDomain()) && !StringUtils.isBlank(dictionaryConfig.fssPrivateDomain().getConfig()) && !dictionaryConfig.fssPrivateDomain().getConfig().startsWith(SystemConstant.START_PARENT)) {
                pdfStringJoiner.add(dictionaryConfig.fssPrivateDomain().getConfig()).add(File.separator);
            }
            pdfStringJoiner = SystemConstant.getDirName(pdfStringJoiner, UploadFileEnum.PDF, true);
            pdfStringJoiner.add(SystemConstant.getNanoId()).add(SystemConstant.PDF_PREFIX);

            File pdfFileTemp = SystemConstant.getFileTempVar(SystemConstant.PDF_PREFIX);
            fileTempList.add(pdfFileTemp);
            createPrintPdfUtil.createSignPdf(pdfFillDto, pdfFileTemp.getPath(), diallel);

            String pdfDirName = pdfStringJoiner.toString().replaceAll("\\\\", SystemConstant.ORG_SPLIT);
            //pdf生成和上传
            String pdfFileMd5 = DigestUtils.md5Hex(new FileInputStream(pdfFileTemp));
            if (oss || (!oss && dictionaryConfig.fssPrivateDomain().getConfig().startsWith(SystemConstant.START_PARENT))) {
                ossStr = oss ? SystemConstant.OSS : SystemConstant.LOCAL;
                fileStoreUtil.ossUpload(pdfDirName, pdfFileTemp, pdfFileMd5, fileStoreUtil.getUploadEnumByPath(pdfDirName).getFssType());
            } else {
                ossStr = SystemConstant.LOCAL;
                fileStoreUtil.localUpload(pdfDirName, new FileInputStream(pdfFileTemp), pdfFileMd5, LocalCatalogEnum.LOCAL_PDF);
            }
            PdfDto pdfDto = PdfUtil.addPdfPage(pdfFileTemp);

            JSONObject attachmentPath = JSONObject.parseObject(examDetail.getAttachmentPath());
            attachmentPath = Objects.isNull(attachmentPath) ? new JSONObject() : attachmentPath;
            JSONArray jsonArray = (JSONArray) attachmentPath.get(SystemConstant.PATH);
            jsonArray = Objects.isNull(jsonArray) ? new JSONArray() : jsonArray;
            JSONObject object = new JSONObject();
            object.put("printType", ClassifyEnum.SIGN.name());
            object.put("pdfMd5", pdfFileMd5);
            object.put(SystemConstant.PDF_PATH, pdfDirName);
            object.put(SystemConstant.TYPE, ossStr);
            object.put(SystemConstant.UPLOAD_TYPE, UploadFileEnum.PDF);
            jsonArray.add(object);
            attachmentPath.put(SystemConstant.PATH, jsonArray);
            examDetail.setAttachmentPath(attachmentPath.toJSONString());
            for (int i = 0; i < printCount; i++) {
                pdfList.add(new PdfDto(pdfFileTemp.getPath(), PageSizeEnum.A4, pdfDto.getPageCount(), 2));
            }
        } catch (Exception e) {
            log.error(SystemConstant.LOG_ERROR, e);
            if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        }
    }

    /**
     * 保存html附件
     *
     * @param fileName
     * @param htmlContent
     * @param userId
     * @param localFileList
     * @param fileTempList
     * @return
     * @throws IOException
     */
    @Override
    @Transactional
    public BasicAttachment saveAttachmentHtml(String fileName, String htmlContent, Long userId, List<PdfDto> localFileList, List<File> fileTempList) throws IOException {
        BasicAttachment basicAttachment = null;
        PdfDto pdfDto = null;
        try {
            File htmlFileTemp = SystemConstant.getFileTempVar(SystemConstant.HTML_PREFIX);
            File pdfFileTemp = SystemConstant.getFileTempVar(SystemConstant.PDF_PREFIX);
            fileTempList.add(htmlFileTemp);
            fileTempList.add(pdfFileTemp);

            FileCopyUtils.copy(htmlContent.getBytes(StandardCharsets.UTF_8), htmlFileTemp);
            boolean oss = dictionaryConfig.sysDomain().isOss();
            StringJoiner stringJoiner = new StringJoiner("");
            StringJoiner pdfStringJoiner = new StringJoiner("");
            if (!oss && Objects.nonNull(dictionaryConfig.fssPublicDomain()) && !StringUtils.isBlank(dictionaryConfig.fssPublicDomain().getConfig()) && !dictionaryConfig.fssPublicDomain().getConfig().startsWith(SystemConstant.START_PARENT)) {
                stringJoiner.add(dictionaryConfig.fssPublicDomain().getConfig()).add(File.separator);
            }
            if (!oss && Objects.nonNull(dictionaryConfig.fssPrivateDomain()) && !StringUtils.isBlank(dictionaryConfig.fssPrivateDomain().getConfig()) && !dictionaryConfig.fssPrivateDomain().getConfig().startsWith(SystemConstant.START_PARENT)) {
                pdfStringJoiner.add(dictionaryConfig.fssPrivateDomain().getConfig()).add(File.separator);
            }
            stringJoiner = SystemConstant.getDirName(stringJoiner, UploadFileEnum.HTML, true);
            stringJoiner.add(SystemConstant.getNanoId()).add(SystemConstant.HTML_PREFIX);

            pdfStringJoiner = SystemConstant.getDirName(pdfStringJoiner, UploadFileEnum.PDF, true);
            pdfStringJoiner.add(SystemConstant.getNanoId()).add(SystemConstant.PDF_PREFIX);

            JSONObject jsonObject = new JSONObject();
            String htmlDirName = stringJoiner.toString().replaceAll("\\\\", SystemConstant.ORG_SPLIT);
            String pdfDirName = pdfStringJoiner.toString().replaceAll("\\\\", SystemConstant.ORG_SPLIT);

            //html生成和上传
            String htmlFileMd5 = DigestUtils.md5Hex(new FileInputStream(htmlFileTemp));
            if (oss || (!oss && dictionaryConfig.fssPrivateDomain().getConfig().startsWith(SystemConstant.START_PARENT))) {
                jsonObject.put(SystemConstant.TYPE, oss ? SystemConstant.OSS : SystemConstant.LOCAL);
                fileStoreUtil.ossUpload(htmlDirName, htmlFileTemp, htmlFileMd5, fileStoreUtil.getUploadEnumByPath(htmlDirName).getFssType());
            } else {
                fileStoreUtil.localUpload(htmlDirName, new FileInputStream(htmlFileTemp), htmlFileMd5, LocalCatalogEnum.LOCAL_FILE);
                jsonObject.put(SystemConstant.TYPE, SystemConstant.LOCAL);
            }
            jsonObject.put(SystemConstant.HTML_PATH, htmlDirName);

            HtmlToPdfUtil.convert(htmlFileTemp.getPath(), pdfFileTemp.getPath(), PageSizeEnum.A3);

            //pdf生成和上传
            String pdfFileMd5 = DigestUtils.md5Hex(new FileInputStream(pdfFileTemp));
            if (oss || (!oss && dictionaryConfig.fssPrivateDomain().getConfig().startsWith(SystemConstant.START_PARENT))) {
                fileStoreUtil.ossUpload(pdfDirName, pdfFileTemp, pdfFileMd5, fileStoreUtil.getUploadEnumByPath(pdfDirName).getFssType());
            } else {
                fileStoreUtil.localUpload(pdfDirName, new FileInputStream(pdfFileTemp), pdfFileMd5, LocalCatalogEnum.LOCAL_PDF);
            }

            pdfDto = PdfUtil.addPdfPage(pdfFileTemp);
            localFileList.add(new PdfDto(pdfFileTemp.getPath(), PageSizeEnum.A3, pdfDto.getPageCount()));
            jsonObject.put(SystemConstant.PATH, pdfDirName);
            jsonObject.put("htmlMd5", htmlFileMd5);
            jsonObject.put("pdfMd5", pdfFileMd5);

            jsonObject.put(SystemConstant.UPLOAD_TYPE, new UploadFileEnum[]{
                    UploadFileEnum.HTML, UploadFileEnum.PDF
            });
            basicAttachment = new BasicAttachment(jsonObject.toJSONString(), fileName, SystemConstant.HTML_PREFIX, new BigDecimal(htmlContent.getBytes(StandardCharsets.UTF_8).length), htmlFileMd5, userId);
            basicAttachment.setPages(pdfDto.getActualPageCount());
            basicAttachmentService.save(basicAttachment);
        } catch (Exception e) {
            log.error(SystemConstant.LOG_ERROR, e);
            basicAttachmentService.deleteAttachment(basicAttachment);
            if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        }
        return basicAttachment;
    }

    /**
     * 保存html附件和该html转成pdf的附件
     *
     * @param fileName    文件名称
     * @param htmlContent html内容
     * @param userId      创建人
     * @return 两个附件对象
     */
    @Transactional
    @Override
    public BasicAttachment saveAttachmentHtmlAndPdf(String fileName, String htmlContent, Long userId) {
        BasicAttachment basicAttachment = null;
        File htmlFile = null, pdfFile = null;
        try {
            byte[] bytes = htmlContent.getBytes(StandardCharsets.UTF_8);
            int size = bytes.length;
            boolean oss = dictionaryConfig.sysDomain().isOss();
            StringJoiner stringJoiner = new StringJoiner("");
            StringJoiner pdfStringJoiner = new StringJoiner("");
            if (!oss && Objects.nonNull(dictionaryConfig.fssPublicDomain()) && !StringUtils.isBlank(dictionaryConfig.fssPublicDomain().getConfig()) && !dictionaryConfig.fssPublicDomain().getConfig().startsWith(SystemConstant.START_PARENT)) {
                stringJoiner.add(dictionaryConfig.fssPublicDomain().getConfig()).add(File.separator);
            }
            if (!oss && Objects.nonNull(dictionaryConfig.fssPrivateDomain()) && !StringUtils.isBlank(dictionaryConfig.fssPrivateDomain().getConfig()) && !dictionaryConfig.fssPrivateDomain().getConfig().startsWith(SystemConstant.START_PARENT)) {
                pdfStringJoiner.add(dictionaryConfig.fssPrivateDomain().getConfig()).add(File.separator);
            }
            stringJoiner = SystemConstant.getDirName(stringJoiner, UploadFileEnum.HTML, true);
            stringJoiner.add(SystemConstant.getNanoId()).add(SystemConstant.HTML_PREFIX);

            pdfStringJoiner = SystemConstant.getDirName(pdfStringJoiner, UploadFileEnum.PDF, true);
            pdfStringJoiner.add(SystemConstant.getNanoId()).add(SystemConstant.PDF_PREFIX);

            String htmlDirName = stringJoiner.toString();
            String pdfDirName = pdfStringJoiner.toString();

            JSONObject jsonObject = new JSONObject();
            jsonObject.put(SystemConstant.PATH, htmlDirName);
            jsonObject.put(SystemConstant.PDF_PATH, pdfDirName);

            htmlFile = SystemConstant.getFileTempVar(SystemConstant.HTML_PREFIX);
            pdfFile = SystemConstant.getFileTempVar(SystemConstant.PDF_PREFIX);

            FileCopyUtils.copy(htmlContent.getBytes(StandardCharsets.UTF_8), htmlFile);

            HtmlToPdfUtil.convert(htmlFile.getPath(), pdfFile.getPath(), PageSizeEnum.A4);
            String htmlFileMd5 = DigestUtils.md5Hex(new FileInputStream(htmlFile));
            String pdfFileMd5 = DigestUtils.md5Hex(new FileInputStream(pdfFile));
            jsonObject.put("htmlMd5", htmlFileMd5);
            jsonObject.put("pdfMd5", pdfFileMd5);
            if (oss || (!oss && dictionaryConfig.fssPrivateDomain().getConfig().startsWith(SystemConstant.START_PARENT))) {//上传至oss
                jsonObject.put(SystemConstant.TYPE, oss ? SystemConstant.OSS : SystemConstant.LOCAL);
                fileStoreUtil.ossUpload(htmlDirName, htmlContent, fileStoreUtil.getUploadEnumByPath(htmlDirName).getFssType());
//                File pdfFile = asposePdfUtil.documentToPdf(localHtmlFile.getPath(), destUrl, PaperSize.A3);
                fileStoreUtil.ossUpload(pdfDirName, pdfFile, pdfFileMd5, fileStoreUtil.getUploadEnumByPath(pdfDirName).getFssType());
            } else {//上传至服务器
                jsonObject.put(SystemConstant.TYPE, SystemConstant.LOCAL);
                fileStoreUtil.localUpload(htmlDirName, new FileInputStream(htmlFile), htmlFileMd5, LocalCatalogEnum.LOCAL_PDF);
            }
            jsonObject.put(SystemConstant.UPLOAD_TYPE, new UploadFileEnum[]{
                    UploadFileEnum.HTML, UploadFileEnum.PDF
            });
            basicAttachment = new BasicAttachment(jsonObject.toJSONString(), fileName, SystemConstant.HTML_PREFIX, new BigDecimal(size), htmlFileMd5, userId);
            basicAttachmentService.save(basicAttachment);
        } catch (Exception e) {
            log.error(SystemConstant.LOG_ERROR, e);
            basicAttachmentService.deleteAttachment(basicAttachment);
            if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        } finally {
            if (Objects.nonNull(htmlFile)) {
                htmlFile.delete();
            }
            if (Objects.nonNull(pdfFile)) {
                pdfFile.delete();
            }
        }
        return basicAttachment;
    }

    /**
     * 保存附件公用
     *
     * @param file
     * @param md5
     * @param type
     * @param objId
     * @return
     */
    @Override
    public BasicAttachment saveAttachmentCommon(MultipartFile file, String md5, UploadFileEnum type, Long objId) {
        Optional.ofNullable(type).orElseThrow(() -> ExceptionResultEnum.ATTACHMENT_TYPE_EMPTY.exception());

        BasicAttachment basicAttachment = null;
        try {
            SysUser requestUser = (SysUser) ServletUtil.getRequestUser();
            String fileName = FilenameUtils.getBaseName(file.getOriginalFilename());
            String format = "." + FilenameUtils.getExtension(file.getOriginalFilename());

            SysConfig sysConfig = commonCacheService.addSysConfigCache(SystemConstant.ATTACHMENT_TYPE);
            Optional.ofNullable(sysConfig).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("未配置附件类型"));
            List<String> attachmentTypeList = Arrays.asList(sysConfig.getConfigValue().replaceAll("\\[", "").replaceAll("\\]", "").split(","));
            if (Objects.nonNull(format)) {
                long count = attachmentTypeList.stream().filter(s -> format.equalsIgnoreCase(s)).count();
                if (count == 0) {
                    throw ExceptionResultEnum.ERROR.exception("文件格式只能为" + attachmentTypeList.toString());
                }
            }

            SysConfig sysConfigLength = commonCacheService.addSysConfigCache(SystemConstant.ATTACHMENT_LENGTH);
            Optional.ofNullable(sysConfigLength).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("未配置附件名称长度"));
            int attachmentLength = Integer.parseInt(sysConfigLength.getConfigValue());
            if (Objects.nonNull(fileName) && fileName.length() > attachmentLength) {
                throw ExceptionResultEnum.ERROR.exception("文件名长度不能超过" + attachmentLength + "个字符");
            }
            long size = file.getSize();
            BigDecimal b = new BigDecimal(size);
            BigDecimal num = new BigDecimal(1024);
            b = b.divide(num, 2, BigDecimal.ROUND_HALF_UP).divide(num, 2, BigDecimal.ROUND_HALF_UP)
                    .setScale(2, BigDecimal.ROUND_HALF_UP);

            SysConfig sysConfigSize = commonCacheService.addSysConfigCache(SystemConstant.ATTACHMENT_SIZE);
            Optional.ofNullable(sysConfigSize).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("未配置附件大小"));
            double attachmentSize = Double.valueOf(sysConfigSize.getConfigValue());
            if (b.doubleValue() > attachmentSize) {
                throw ExceptionResultEnum.ERROR.exception("文件大小不能超过" + attachmentSize + "MB");
            }
            log.info("fileName:{}", fileName);
            log.info("format:{}", format);
            log.info("size:{}", b);
            log.info("getOriginalFilename:{}", file.getOriginalFilename());
            String fileMd5 = DigestUtils.md5Hex(file.getBytes());
            log.info("fileMd5:{}", fileMd5);
            log.info("md5:{}", md5);
            if (!Objects.equals(fileMd5, md5)) {
                throw ExceptionResultEnum.MD5_EQUALS_FALSE.exception();
            }
            boolean oss = dictionaryConfig.sysDomain().isOss();
            StringJoiner stringJoiner = new StringJoiner("");
            if (!oss && (Objects.nonNull(dictionaryConfig.fssPublicDomain()) && !StringUtils.isBlank(dictionaryConfig.fssPublicDomain().getConfig())) && !dictionaryConfig.fssPublicDomain().getConfig().startsWith(SystemConstant.START_PARENT)) {
                stringJoiner.add(dictionaryConfig.fssPublicDomain().getConfig()).add(File.separator);
            }
            stringJoiner = SystemConstant.getDirName(stringJoiner, type, true);
            stringJoiner.add(SystemConstant.getNanoId()).add(format);

            JSONObject jsonObject = new JSONObject();
            String dirName = stringJoiner.toString().replaceAll("\\\\", SystemConstant.ORG_SPLIT);
            if (oss || (!oss && dictionaryConfig.fssPrivateDomain().getConfig().startsWith(SystemConstant.START_PARENT))) {//上传至oss
                fileStoreUtil.ossUpload(dirName, file.getInputStream(), md5, type.getFssType());
                jsonObject.put(SystemConstant.TYPE, oss ? SystemConstant.OSS : SystemConstant.LOCAL);
                jsonObject.put(SystemConstant.PATH, dirName);
            } else {//上传至服务器
                fileStoreUtil.localUpload(dirName, file.getInputStream(), md5, LocalCatalogEnum.LOCAL_FILE);
                jsonObject.put(SystemConstant.TYPE, SystemConstant.LOCAL);
                jsonObject.put(SystemConstant.PATH, dirName);
            }
            jsonObject.put(SystemConstant.UPLOAD_TYPE, type);

            basicAttachment = new BasicAttachment(jsonObject.toJSONString(), fileName, format, b, fileMd5, requestUser.getId(), objId);
            basicAttachmentService.save(basicAttachment);

            // pdf需要读取总页数
            Integer pages = 0;
            if (SystemConstant.PDF_PREFIX.equals(format)) {
                PdfReader pdfReader = new PdfReader(file.getBytes());
                pages = pdfReader.getNumberOfPages();
            }
            basicAttachment.setPages(pages);
        } catch (Exception e) {
            log.error(SystemConstant.LOG_ERROR, e);
            basicAttachmentService.deleteAttachment(basicAttachment);
            if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        }
        return basicAttachment;
    }

    /**
     * 保存附件
     *
     * @param file
     * @param md5
     * @param type
     * @return
     */
    @Override
    @Transactional
    public BasicAttachment saveAttachment(MultipartFile file, String md5, UploadFileEnum type) {
        return saveAttachmentCommon(file, md5, type, null);
    }

    /**
     * 保存附件
     *
     * @param file
     * @param md5
     * @param type
     * @param objId
     * @return
     */
    @Override
    @Transactional
    public BasicAttachment saveAttachment(MultipartFile file, String md5, UploadFileEnum type, Long objId) {
        return saveAttachmentCommon(file, md5, type, objId);
    }

    /**
     * 科目删除
     *
     * @param id
     * @return
     */
    @Override
    public boolean remove(Long id) {
        // code是否已有命题任务
        BasicCourse course = basicCourseMapper.selectById(id);
        List<ExamTask> taskList = examTaskService.listByCourseCode(course.getSchoolId(), course.getCode());
        if (taskList != null && taskList.size() > 0) {
            throw ExceptionResultEnum.ERROR.exception("课程绑定命题任务，不能删除");
        }
        return basicCourseService.removeById(id);
    }

    /**
     * 分页查询
     *
     * @param code
     * @param name
     * @param pageNumber
     * @param pageSize
     * @return
     */
    @Override
    public IPage<BasicCourse> list(String code, String name, Integer pageNumber, Integer pageSize) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        code = SystemConstant.translateSpecificSign(code);
        name = SystemConstant.translateSpecificSign(name);
        Page<BasicCourse> page = new Page<>(pageNumber, pageSize);
        IPage<BasicCourse> courseIPage = basicCourseMapper.listPage(page, schoolId, SystemConstant.translateSpecificSign(code), SystemConstant.translateSpecificSign(name));
        return courseIPage;
    }

    /**
     * 科目查询
     *
     * @param param
     * @param printPlanIdList
     * @return
     */
    @Override
    public List<BasicCourse> list(String param, List<Long> printPlanIdList, Long teachingRoomId) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        SysUser requestUser = (SysUser) ServletUtil.getRequestUser();
        SysOrg sysOrg = sysOrgService.getById(requestUser.getOrgId());
        Set<Long> orgIds = teachcloudCommonService.listSubOrgIds(teachingRoomId);
//        DataPermissionRule dpr = basicRoleDataPermissionService.findDataPermission(schoolId, requestUser.getId(), ServletUtil.getRequest().getServletPath());
        /*if (Objects.nonNull(teachingRoomId)) {
            orgIds = new HashSet<>(Arrays.asList(teachingRoomId));
        } else {
            orgIds = teachcloudCommonService.listSubOrgIds(teachingRoomId);
        }*/
        if (CollectionUtils.isEmpty(printPlanIdList)) {
            QueryWrapper<BasicCourse> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(BasicCourse::getSchoolId, schoolId)
                    .eq(BasicCourse::getEnable, true);
            if (sysOrg != null && !OrgTypeEnum.PRINTING_HOUSE.equals(sysOrg.getType())) {
                queryWrapper.lambda().in(BasicCourse::getTeachingRoomId, orgIds);
            }
            if (StringUtils.isNotBlank(param)) {
                queryWrapper.lambda().and(i -> i.like(BasicCourse::getCode, SystemConstant.translateSpecificSign(param))
                        .or().like(BasicCourse::getName, SystemConstant.translateSpecificSign(param)));
            }
            return basicCourseService.list(queryWrapper);
        } else {
            return examDetailCourseService.listCoursesByPrintPlanId(param, printPlanIdList);
        }
    }

    /**
     * 机构禁用
     *
     * @param org
     * @return
     */
    @Override
    public boolean enable(SysOrg org) {
        // 禁用时，校验机构
        if (!org.getEnable()) {
            validateOrg(org.getId());
        }
        UpdateWrapper<SysOrg> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(SysOrg::getEnable, org.getEnable()).eq(SysOrg::getId, org.getId());
        return sysOrgService.update(updateWrapper);
    }

    /**
     * 机构删除
     *
     * @param id
     * @return
     */
    @Override
    public boolean sysOrgRemove(Long id) {
        // 机构校验
        validateOrg(id);
        boolean result = sysOrgService.removeById(id);
        Long schoolId = SystemConstant.convertIdToLong(String.valueOf(ServletUtil.getRequestHeaderSchoolId()));
        commonCacheService.removeOrgCollegeLevelCache(schoolId);
        return result;
    }

    private void validateOrg(Long id) {
        // 机构下是否有用户
        List<SysUser> sysUsers = sysUserService.listByOrgId(id);
        if (sysUsers != null && sysUsers.size() > 0) {
            throw ExceptionResultEnum.ERROR.exception("该机构下有用户，不能删除");
        }

        // 机构下是否有子机构
        QueryWrapper<SysOrg> orgQueryWrapper = new QueryWrapper<>();
        orgQueryWrapper.lambda().eq(SysOrg::getParentId, id);
        List<SysOrg> sysOrgs = sysOrgService.list(orgQueryWrapper);
        if (sysOrgs != null && sysOrgs.size() > 0) {
            throw ExceptionResultEnum.ERROR.exception("该机构下有子机构，不能删除");
        }

//        QueryWrapper<BasicTemplateOrg> queryWrapper = new QueryWrapper<>();
//        queryWrapper.lambda().eq(BasicTemplateOrg::getOrgId, id);
//        List<BasicTemplateOrg> basicTemplateOrgs = basicTemplateOrgService.list(queryWrapper);
//        if (basicTemplateOrgs != null && basicTemplateOrgs.size() > 0) {
//            throw ExceptionResultEnum.ERROR.exception("机构已绑定，不能删除");
//        }
    }

    /**
     * 更新任务状态
     */
    @Override
    @Transactional
    public void updateStatus() throws IOException {
        //不是自动生成pdf的全部改为失败
//        UpdateWrapper<TBTask> tbTaskUpdateWrapper = new UpdateWrapper<>();
//        tbTaskUpdateWrapper.lambda().set(TBTask::getStatus, TaskStatusEnum.FINISH)
//                .set(TBTask::getSummary, "系统提交:网络异常或程序异常中断!")
//                .set(TBTask::getResult, TaskResultEnum.ERROR)
//                .isNotNull(TBTask::getStatus).ne(TBTask::getStatus, TaskStatusEnum.FINISH)
//                .ne(TBTask::getType, TaskTypeEnum.CREATE_PDF);
//        this.update(tbTaskUpdateWrapper);

        QueryWrapper<TBTask> tbTaskQueryWrapper = new QueryWrapper<>();
        tbTaskQueryWrapper.lambda().eq(TBTask::getType, TaskTypeEnum.CREATE_PDF)
                .ne(TBTask::getStatus, TaskStatusEnum.FINISH)
                .ne(TBTask::getResult, TaskResultEnum.SUCCESS).or().isNull(TBTask::getResult)
                .orderByDesc(TBTask::getStatus);
        List<TBTask> tbTasks = tbTaskService.list(tbTaskQueryWrapper);
        if (Objects.nonNull(tbTasks) && tbTasks.size() > 0) {
            for (TBTask tbTask : tbTasks) {
                Map<String, Object> map = new HashMap<>();
                if (Objects.nonNull(tbTask.getRemark())) {
                    JSONArray jsonArray = JSONArray.parseArray(tbTask.getRemark());
                    ArraysParams arraysParams = new ArraysParams(jsonArray.toArray(new Long[jsonArray.size()]));
                    map.computeIfAbsent("examDetailCourseIds", v -> Arrays.asList(arraysParams.getIds()));
                }
                map.computeIfAbsent(SystemConstant.TASK, v -> tbTask);
                map.computeIfAbsent(SystemConstant.USER, v -> sysUserService.getById(tbTask.getCreateId()));
//                asyncCreatePdfTempleteService.createPdf(map, null);
                MqDto mqDto = new MqDto(MqTagEnum.PDF.getCode(), map, String.valueOf(tbTask.getId()));
                redisUtil.sendMessage(mqDto.getTopic(), mqDto);
            }
        }
    }

    /**
     * 重新生成pdf失败任务
     *
     * @throws IOException
     */
    @Override
    @Transactional
    public void resetCreatePdfTask() throws IOException {
        QueryWrapper<TBTask> tbTaskQueryWrapper = new QueryWrapper<>();
        tbTaskQueryWrapper.lambda().eq(TBTask::getType, TaskTypeEnum.CREATE_PDF)
                .eq(TBTask::getStatus, TaskStatusEnum.FINISH).isNotNull(TBTask::getStatus)
                .eq(TBTask::getResult, TaskResultEnum.ERROR).last("limit 0,100");
        List<TBTask> tbTasks = tbTaskService.list(tbTaskQueryWrapper);
        if (Objects.nonNull(tbTasks) && tbTasks.size() > 0) {
            for (TBTask tbTask : tbTasks) {
                SysConfig sysConfig = commonCacheService.addSysConfigCache(SystemConstant.AUTO_CREATE_PDF_RESET_MAX_COUNT);
                Optional.ofNullable(sysConfig).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("未配置自动重试生成pdf失败最大次数"));
                Integer maxResetCount = Integer.parseInt(sysConfig.getConfigValue());
                if (tbTask.getResetCount() < maxResetCount.intValue()) {
                    Map<String, Object> map = new HashMap<>();
//                    tbTask.setVersion(new AtomicInteger(tbTask.getVersion()).incrementAndGet());
                    if (Objects.nonNull(tbTask.getRemark())) {
                        JSONArray jsonArray = JSONArray.parseArray(tbTask.getRemark());
                        ArraysParams arraysParams = new ArraysParams(jsonArray.toArray(new Long[jsonArray.size()]));
                        map.computeIfAbsent("examDetailCourseIds", v -> Arrays.asList(arraysParams.getIds()));
                    }
                    map.computeIfAbsent(SystemConstant.TASK, v -> tbTask);
                    map.computeIfAbsent(SystemConstant.USER, v -> sysUserService.getById(tbTask.getCreateId()));
//                    asyncCreatePdfTempleteService.createPdf(map, null);
                    MqDto mqDto = new MqDto(MqTagEnum.PDF.getCode(), map, String.valueOf(tbTask.getId()));
                    redisUtil.sendMessage(mqDto.getTopic(), mqDto);
                }
            }
        }
    }

    /**
     * 保存任务
     *
     * @param file
     * @param taskTypeEnum
     * @return
     */
    @Override
    @Transactional
    public Map<String, Object> saveTask(MultipartFile file, TaskTypeEnum taskTypeEnum) {
        BasicAttachment basicAttachment = null;
        Map<String, Object> map = null;
        try {
            PrintCommonService printCommonService = SpringContextHolder.getBean(PrintCommonService.class);
            basicAttachment = printCommonService.saveAttachment(file, ServletUtil.getRequestMd5(), UploadFileEnum.FILE);
            if (Objects.isNull(basicAttachment)) {
                throw ExceptionResultEnum.ATTACHMENT_ERROR.exception();
            } else {
                map = new HashMap<>();
                TBTask tbTask = tbTaskService.saveTaskCommon(basicAttachment, taskTypeEnum, map, null);
                tbTaskService.save(tbTask);
            }
        } catch (Exception e) {
            log.error(SystemConstant.LOG_ERROR, e);
            if (Objects.nonNull(basicAttachment)) {
                basicAttachmentService.deleteAttachment(basicAttachment);
            }
            if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        }
        return map;
    }

    @Override
    @Transactional
    public Map<String, Object> saveTask(Long printPlanId, MultipartFile file, TaskTypeEnum taskTypeEnum) {
        BasicAttachment basicAttachment = null;
        Map<String, Object> map = null;
        try {
            PrintCommonService printCommonService = SpringContextHolder.getBean(PrintCommonService.class);
            basicAttachment = printCommonService.saveAttachment(file, ServletUtil.getRequestMd5(), UploadFileEnum.FILE);
            if (Objects.isNull(basicAttachment)) {
                throw ExceptionResultEnum.ATTACHMENT_ERROR.exception();
            } else {
                map = new HashMap<>();
                TBTask tbTask = tbTaskService.saveTaskCommon(basicAttachment, taskTypeEnum, map, null);
                tbTask.setPrintPlanId(printPlanId);
                tbTaskService.save(tbTask);
            }
        } catch (Exception e) {
            log.error(SystemConstant.LOG_ERROR, e);
            if (Objects.nonNull(basicAttachment)) {
                basicAttachmentService.deleteAttachment(basicAttachment);
            }
            if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        }
        return map;
    }

    @Override
    @Transactional
    public Map<String, Object> saveTask(TaskTypeEnum taskTypeEnum) {
        Map<String, Object> map = null;
        try {
            map = new HashMap<>();
            TBTask tbTask = tbTaskService.saveTaskCommon(null, taskTypeEnum, map, null);
            tbTaskService.save(tbTask);
        } catch (Exception e) {
            log.error(SystemConstant.LOG_ERROR, e);
            if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        }
        return map;
    }

    @Override
    @Transactional
    public Map<String, Object> saveTask(Long printPlanId, TaskTypeEnum taskTypeEnum) {
        Map<String, Object> map = null;
        try {
            map = new HashMap<>();
            TBTask tbTask = tbTaskService.saveTaskCommon(null, taskTypeEnum, map, null);
            tbTask.setPrintPlanId(printPlanId);
            tbTaskService.save(tbTask);
        } catch (Exception e) {
            log.error(SystemConstant.LOG_ERROR, e);
            if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        }
        return map;
    }

    @Transactional
    @Override
    public Map<String, Object> savePush(PushTypeEnum pushTypeEnum) {
        Map<String, Object> map = null;
        try {
            map = new HashMap<>();
            TBSyncTask tbSyncTask = tbSyncTaskService.savePushCommon(pushTypeEnum, map, null);
            tbSyncTaskService.save(tbSyncTask);
        } catch (Exception e) {
            log.error(SystemConstant.LOG_ERROR, e);
            if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        }
        return map;
    }

    @Transactional
    @Override
    public Map<String, Object> savePush(PushTypeEnum pushTypeEnum, Long objectId) {
        Map<String, Object> map = null;
        try {
            // 校验任务是否进行中
            QueryWrapper<TBSyncTask> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(TBSyncTask::getType, pushTypeEnum).eq(TBSyncTask::getObjectId, objectId).ne(TBSyncTask::getStatus, TaskStatusEnum.FINISH);
            List<TBSyncTask> list = tbSyncTaskService.list(queryWrapper);
            if (!list.isEmpty()) {
                throw ExceptionResultEnum.ERROR.exception("当前同步条件下有任务正在进行中");
            }
            map = new HashMap<>();
            TBSyncTask tbSyncTask = tbSyncTaskService.savePushCommon(pushTypeEnum, map, null, objectId);
            tbSyncTaskService.save(tbSyncTask);
        } catch (Exception e) {
            log.error(SystemConstant.LOG_ERROR, e);
            if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        }
        return map;
    }

    /**
     * 校验课程关联考场是否提交打印
     *
     * @param schoolId
     * @param printPlanId
     * @param courseCode
     * @param paperNumber
     * @return
     */
    @Override
    public Boolean checkExamDetailStatus(Long schoolId, Long printPlanId, String courseCode, String paperNumber) {
        List<ExamDetail> examDetails;
        ExamPrintPlan examPrintPlan = examPrintPlanService.getById(printPlanId);
        if (printPlanId != null) {
            examDetails = examDetailService.listByPrintPlanIdAndCourseCodeAndPaperNumber(schoolId, printPlanId, courseCode, paperNumber);
        } else {
            examDetails = examDetailService.listByCourseCodeAndPaperNumber(schoolId, examPrintPlan.getExamId(), courseCode, paperNumber);
        }
        if (examDetails != null && examDetails.size() > 0) {
            long count = examDetails.stream().filter(m -> !ExamDetailStatusEnum.NEW.name().equals(m.getStatus().name()) && !ExamDetailStatusEnum.READY.name().equals(m.getStatus().name())).count();
            if (count > 0) {
                throw ExceptionResultEnum.ERROR.exception("已有印刷任务进入打印状态，不能修改");
            }
        }
        return true;
    }

    /**
     * 生成唯一的试卷编号
     *
     * @param schoolId
     * @return
     */
    @Override
    public String createPaperNumber(Long schoolId, Long examId) {
        String date = DateUtil.today().replace(SystemConstant.HYPHEN, "");
        String paperNumber = convertUtil.getIncre(date, "paperNumber" + schoolId, 5);
        QueryWrapper<ExamTask> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ExamTask::getSchoolId, schoolId).eq(ExamTask::getPaperNumber, paperNumber);
        ExamTask examTask = examTaskService.getOne(queryWrapper);
        if (examTask != null) {
            return createPaperNumber(schoolId, examId);
        } else {
            return paperNumber;
        }
    }

    /**
     * 校验是否可以提交
     *
     * @param schoolId
     * @param courseCode
     * @param paperNumber
     * @param user
     * @throws IOException
     */
    @Override
    public void checkData(Long schoolId, Long examId, String courseCode, String paperNumber, SysUser user) throws IOException {
        // 校验命题任务是否提交
        ExamTask examTask = examTaskService.getByCourseCodeAndPaperNumber(schoolId, examId, courseCode, paperNumber);
        BasicExam basicExam = basicExamService.getById(examTask.getExamId());
        if (basicExam != null && ExamModelEnum.MODEL3.equals(basicExam.getExamModel())) {
            log.info("考试模式为模式3，不生成pdf");
            return;
        }

        if (examTask != null && !examTask.getEnable()) {
            log.info("命题任务已禁用");
            return;
        }
        ExamTaskDetail examTaskDetail = examTaskDetailService.getByExamTaskId(examTask.getId());
        if (!examTaskDetail.getEnable()) {
            log.info("卷库已禁用");
            return;
        }
        TFFlowApprove tfFlowApprove = tfFlowApproveService.findByFlowId(examTask.getFlowId());
        // 1.命题任务已完成
        if (tfFlowApprove != null && tfFlowApprove.getStatus() == FlowStatusEnum.FINISH) {
            // 2.校验考务数据是否导入
            List<ExamDetail> examDetails = examDetailService.listByCourseCodeAndPaperNumber(schoolId, examId, courseCode, paperNumber);
            if (examDetails != null && examDetails.size() > 0) {

                // 3.检查examDetailId下有无其它课程
                for (ExamDetail examDetail : examDetails) {
                    List<ExamDetailCourse> examDetailCourses = examDetailCourseService.listByExamDetailIdAndStatus(examDetail.getId());
                    // 4.没有未完成的命题任务
                    if (examDetailCourses.isEmpty()) {
                        // 更新考场状态为初始新建状态
                        UpdateWrapper<ExamDetail> examDetailUpdateWrapper = new UpdateWrapper<>();
                        examDetailUpdateWrapper.lambda().set(ExamDetail::getStatus, ExamDetailStatusEnum.NEW).eq(ExamDetail::getId, examDetail.getId());
                        examDetailService.update(examDetailUpdateWrapper);

                        //所有考场都撤回，印刷任务状态改为就绪
                        QueryWrapper<ExamDetail> queryWrapper = new QueryWrapper<>();
                        queryWrapper.lambda().eq(ExamDetail::getPrintPlanId, examDetail.getPrintPlanId()).notIn(ExamDetail::getStatus, ExamDetailStatusEnum.NEW, ExamDetailStatusEnum.READY);
                        List<ExamDetail> examDetailss = examDetailService.list(queryWrapper);
                        if (examDetailss.isEmpty()) {
                            UpdateWrapper<ExamPrintPlan> printPlanUpdateWrapper = new UpdateWrapper<>();
                            printPlanUpdateWrapper.lambda().set(ExamPrintPlan::getStatus, PrintPlanStatusEnum.READY).eq(ExamPrintPlan::getId, examDetail.getPrintPlanId());
                            examPrintPlanService.update(printPlanUpdateWrapper);
                        }

                        Map<String, Object> map = tbTaskService.saveTask(TaskTypeEnum.CREATE_PDF, examDetail.getPrintPlanId(), user, examDetail.getId());
//                        asyncCreatePdfTempleteService.createPdf(map, null);
                        MqDto mqDto = new MqDto(MqTagEnum.PDF.getCode(), map, String.valueOf(map.get(SystemConstant.TB_TASK_ID)));
                        redisUtil.sendMessage(mqDto.getTopic(), mqDto);
                    }
                }
            }
        }
    }

    @Override
    public List<ExamStudent> createBatchStudent(Long schoolId, String extendFields, List<String> classIds, Long examDetailCourseId, SysUser sysUser) {
        AtomicInteger atomicInteger = new AtomicInteger(1);
        List<ExamStudent> examStudentAll = new ArrayList<>();
        SerialNumberParams ticketNumberParams = new SerialNumberParams("ticketNumber-" + schoolId, DateUtil.format(new Date(), "yyyyMM"), 6);

        for (String classId : classIds) {
            List<ExamStudent> basicStudents = examStudentService.listExamStudentBySchoolIdAndClazzId(schoolId, classId);

            basicStudents = basicStudents.stream().map(examStudent -> {
                examStudent.setId(SystemConstant.getDbUuid());
                examStudent.setSchoolId(schoolId);
                examStudent.setOrgId(sysUser.getOrgId());
                examStudent.setExamDetailCourseId(examDetailCourseId);
                // 准考证号（年月+000001）
                examStudent.setTicketNumber(createTempNumber(ticketNumberParams));
                examStudent.setExtendFields(extendFields);
                examStudent.setSiteNumber(String.valueOf(atomicInteger.getAndIncrement()));
                examStudent.setCreateId(sysUser.getId());
                examStudent.setPaperNumber(examDetailCourseService.getById(examDetailCourseId).getPaperNumber());
                return examStudent;
            }).collect(Collectors.toList());
            if (!CollectionUtils.isEmpty(basicStudents)) {
                examStudentAll.addAll(basicStudents);
            }
        }

        return examStudentAll;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public List<ExamStudent> createBatchStudentByStudentList(Long schoolId, Long examId, String semesterId, String paperNumber, String extendFields, List<ExamTaskStudentObjectParam> examTaskStudentObjectParamList, Long examDetailCourseId, String examShortCode, String courseShortCode, Long cardRuleId, SysUser sysUser) {
        AtomicInteger atomicInteger = new AtomicInteger(1);

        BasicCardRule basicCardRule = basicCardRuleService.getById(cardRuleId);

        Map<String, String> map = new HashMap<>();
        List<ExamStudent> examStudentList = new ArrayList<>();
        for (ExamTaskStudentObjectParam student : examTaskStudentObjectParamList) {
            Long studentId = student.getStudentId();
            Long teachClazzId = student.getTeachClazzId();
            Long basicClazzId = student.getBasicClazzId();
            StudentClazzEnum studentClazzType = student.getStudentClazzType();

            BasicStudentResult basicStudentResult = basicStudentService.findById(studentId);
            if (Objects.isNull(basicStudentResult)) {
                throw ExceptionResultEnum.ERROR.exception("未找到基础学生信息");
            }
            if (!Objects.equals(basicStudentResult.getClazzId(), basicClazzId)) {
                throw ExceptionResultEnum.ERROR.exception("行政班数据不一致");
            }
            BasicClazz basicClazz = null;
            if (SystemConstant.longNotNull(basicClazzId)) {
                basicClazz = basicClazzService.getById(basicClazzId);
                if (Objects.isNull(basicClazz)) {
                    throw ExceptionResultEnum.ERROR.exception("未找到行政班");
                }
                map.putIfAbsent(String.valueOf(basicClazzId), basicClazz.getShortCode());
            }

            TeachClazz teachClazz = null;
            if (SystemConstant.longNotNull(teachClazzId)) {
                teachClazz = teachClazzService.getById(teachClazzId);
                if (Objects.isNull(teachClazz)) {
                    throw ExceptionResultEnum.ERROR.exception("未找到教学班");
                }
            }

            ExamStudent examStudent = new ExamStudent();
            examStudent.setSchoolId(schoolId);
            examStudent.setId(SystemConstant.getDbUuid());
            examStudent.setOrgId(sysUser.getOrgId());
            examStudent.setExamDetailCourseId(examDetailCourseId);
            examStudent.setStudentName(basicStudentResult.getStudentName());
            examStudent.setStudentCode(basicStudentResult.getStudentCode());
            examStudent.setExamId(examId);
            examStudent.setPaperNumber(paperNumber);
            examStudent.setCollegeName(basicStudentResult.getCollegeName());
            examStudent.setMajorName(basicStudentResult.getMajorName());
            examStudent.setTicketNumber(createTicketNumber(schoolId, basicCardRule == null ? null : basicCardRule.getExamNumberDigit()));
            examStudent.setSiteNumber(String.valueOf(atomicInteger.getAndIncrement()));
            examStudent.setExtendFields(extendFields);
            examStudent.setCreateId(sysUser.getId());
            // 行政班数据
            if (Objects.nonNull(basicClazz)) {
                examStudent.setClazzId(String.valueOf(basicClazz.getId()));
                examStudent.setClazzName(basicClazz.getClazzName());
            }
            // 教学班模式 教学班数据
            if (Objects.nonNull(teachClazz)) {
                examStudent.setTeachClazzId(teachClazz.getId());
                examStudent.setTeachClazzName(teachClazz.getTeachClazzName());
            }
            examStudent.setStudentClazzType(studentClazzType);
            examStudentList.add(examStudent);
        }
        return examStudentList;
    }

    @Override
    public String createTempNumber(SerialNumberParams serialNumberParams) {
        return convertUtil.getIncre(serialNumberParams.getPrefix(), serialNumberParams.getModel(), serialNumberParams.getDigit());
    }


    /**
     * 生成唯一的准考证号
     *
     * @param schoolId   学校ID
     * @param totalDigit 考号位数
     * @return
     */
    @Override
    public String createTicketNumber(Long schoolId, Integer totalDigit) {
        if (totalDigit == null) {
            totalDigit = 10;
        }
        try {
            try {
                lock.tryLock(30, TimeUnit.SECONDS);
            } catch (InterruptedException e) {
                log.error("生成准考证号失败");
            }
            String date = DateUtil.today().replace(SystemConstant.HYPHEN, "");
            String prefix;
            if (totalDigit < 10) {
                prefix = date.substring(2, 4);
            } else {
                prefix = date.substring(2);
            }
            return convertUtil.getIncre(prefix, SystemConstant.NUMBER_CACHE + "ticketNumber" + schoolId, totalDigit - prefix.length());

        } finally {
            lock.unlock();
        }
    }

    /**
     * 保存任务附件(导出)
     *
     * @param fos
     * @param oss
     * @return
     */
    @Override
    public String saveTaskAttachment(ByteArrayOutputStream fos, boolean oss) throws IOException {
        InputStream inputStream = null;
        JSONObject jsonObject = null;
        try {
            inputStream = new ByteArrayInputStream(fos.toByteArray());
            StringJoiner stringJoiner = new StringJoiner("");
            if (!oss && Objects.nonNull(dictionaryConfig.fssPublicDomain()) && !StringUtils.isBlank(dictionaryConfig.fssPublicDomain().getConfig()) && !dictionaryConfig.fssPublicDomain().getConfig().startsWith(SystemConstant.START_PARENT)) {
                stringJoiner.add(dictionaryConfig.fssPublicDomain().getConfig()).add(File.separator);
            }
            stringJoiner = SystemConstant.getDirName(stringJoiner, UploadFileEnum.FILE, true);
            stringJoiner.add(SystemConstant.getNanoId()).add(".").add(SystemConstant.XLSX);

            jsonObject = new JSONObject();
            String dirName = stringJoiner.toString().replaceAll("\\\\", SystemConstant.ORG_SPLIT);
            if (oss || (!oss && dictionaryConfig.fssPrivateDomain().getConfig().startsWith(SystemConstant.START_PARENT))) {//上传至oss
                fileStoreUtil.ossUpload(dirName, inputStream, DigestUtils.md5Hex(new ByteArrayInputStream(fos.toByteArray())), UploadFileEnum.FILE.getFssType());
                jsonObject.put(SystemConstant.TYPE, oss ? SystemConstant.OSS : SystemConstant.LOCAL);
            } else {
                fileStoreUtil.copyInputStreamToFile(inputStream, new File(stringJoiner.toString()), DigestUtils.md5Hex(new ByteArrayInputStream(fos.toByteArray())), LocalCatalogEnum.LOCAL_FILE);
                jsonObject.put(SystemConstant.TYPE, SystemConstant.LOCAL);
            }
            jsonObject.put(SystemConstant.PATH, dirName);
            jsonObject.put(SystemConstant.UPLOAD_TYPE, UploadFileEnum.FILE);
        } catch (Exception e) {
            log.error(SystemConstant.LOG_ERROR, e);
        } finally {
            if (Objects.nonNull(inputStream)) {
                inputStream.close();
            }
        }
        return Objects.nonNull(jsonObject) ? jsonObject.toJSONString() : null;
    }

    @Override
    public String createCourseSequence(Long schoolId, String courseCode) {
        boolean lock = redisUtil.lock(SystemConstant.REDIS_LOCK_COURSE_CODE_SEQUENCE_PREFIX + schoolId + "-" + courseCode, SystemConstant.REDIS_LOCK_PAPER_NUMBER_TIME_OUT);
        if (!lock) {
            throw ExceptionResultEnum.ERROR.exception("正在生成课程编号中，请稍候再试！");
        }
        try {
            String sequence = commonCacheService.addExamTaskSequence(schoolId, courseCode);
            if ("0".equals(sequence)) {
                QueryWrapper<ExamTask> queryWrapper = new QueryWrapper<>();
                queryWrapper.lambda().eq(ExamTask::getSchoolId, schoolId).eq(ExamTask::getCourseCode, courseCode);
                List<ExamTask> examTasks = examTaskService.list(queryWrapper);
                if (examTasks != null && examTasks.size() > 0) {
                    String maxSequence = examTasks.stream().max(Comparator.comparing(ExamTask::getSequence)).get().getSequence();
                    if (StringUtils.isNotBlank(maxSequence)) {
                        sequence = maxSequence;
                    }
                }
            }
            return commonCacheService.updateExamTaskSequence(schoolId, courseCode, sequence);
        } finally {
            redisUtil.releaseCourseSequenceLock(SystemConstant.REDIS_LOCK_COURSE_CODE_SEQUENCE_PREFIX + schoolId + "-" + courseCode);
        }
    }

    @Override
    public void updateGradeBatchStatus(Long schoolId, Long examId, String paperNumber, String paperType) {
        QueryWrapper<GradeBatchPaper> gradeBatchPaperQueryWrapper = new QueryWrapper<>();
        gradeBatchPaperQueryWrapper.lambda()
                .eq(GradeBatchPaper::getSchoolId, schoolId)
                .eq(GradeBatchPaper::getExamId, examId)
                .eq(GradeBatchPaper::getPaperNumber, paperNumber)
                .eq(GradeBatchPaper::getPaperType, paperType);
        List<GradeBatchPaper> gradeBatchPaperList = gradeBatchPaperService.list(gradeBatchPaperQueryWrapper);
        if (!gradeBatchPaperList.isEmpty()) {
            for (GradeBatchPaper gradeBatchPaper : gradeBatchPaperList) {
                QueryWrapper<GradeBatchPaper> queryWrapperList = new QueryWrapper<>();
                queryWrapperList.lambda().eq(GradeBatchPaper::getBatchId, gradeBatchPaper.getBatchId()).eq(GradeBatchPaper::getExamId, examId);
                List<GradeBatchPaper> gradeBatchPaperAll = gradeBatchPaperService.list(queryWrapperList);
                if (!gradeBatchPaperAll.isEmpty()) {
                    long count = gradeBatchPaperAll.stream().filter(m -> !GradeAnalyzePaperStatusEnum.READY_TO_CALCULATE.equals(m.getStatus())).count();
                    if (count == 0) {
                        // 更新批次状态
                        UpdateWrapper<GradeBatch> updateWrapper = new UpdateWrapper<>();
                        updateWrapper.lambda().set(GradeBatch::getStatus, GradeAnalyzePaperStatusEnum.READY_TO_CALCULATE).eq(GradeBatch::getId, gradeBatchPaper.getBatchId());
                    }
                }
            }
        }
    }

    /**
     * 生成卷袋贴
     *
     * @param pdfPackageDto
     * @param examDetail
     * @param pdfList
     * @param printCount
     * @param fileTempList
     */
    @Override
    public void saveAttachmentPackagePdf(PdfPackageDto pdfPackageDto, ExamDetail examDetail, List<PdfDto> pdfList, Integer printCount, List<File> fileTempList) {
        try {
            boolean oss = dictionaryConfig.sysDomain().isOss();
            StringJoiner pdfStringJoiner = new StringJoiner("");
            String ossStr = null;
            if (!oss && Objects.nonNull(dictionaryConfig.fssPrivateDomain()) && !StringUtils.isBlank(dictionaryConfig.fssPrivateDomain().getConfig()) && !dictionaryConfig.fssPrivateDomain().getConfig().startsWith(SystemConstant.START_PARENT)) {
                pdfStringJoiner.add(dictionaryConfig.fssPrivateDomain().getConfig()).add(File.separator);
            }
            pdfStringJoiner = SystemConstant.getDirName(pdfStringJoiner, UploadFileEnum.PDF, true);
            pdfStringJoiner.add(SystemConstant.getNanoId()).add(SystemConstant.PDF_PREFIX);

            String pdfDirName = pdfStringJoiner.toString().replaceAll("\\\\", SystemConstant.ORG_SPLIT);
            File pdfFileTemp = SystemConstant.getFileTempVar(SystemConstant.PDF_PREFIX);
            fileTempList.add(pdfFileTemp);
            createPrintPdfUtil.createPackagePdf(pdfPackageDto, pdfFileTemp.getPath());
            //pdf生成和上传
            String pdfFileMd5 = DigestUtils.md5Hex(new FileInputStream(pdfFileTemp));
            if (oss || (!oss && dictionaryConfig.fssPrivateDomain().getConfig().startsWith(SystemConstant.START_PARENT))) {
                ossStr = oss ? SystemConstant.OSS : SystemConstant.LOCAL;
                fileStoreUtil.ossUpload(pdfDirName, pdfFileTemp, pdfFileMd5, fileStoreUtil.getUploadEnumByPath(pdfDirName).getFssType());
            } else {
                ossStr = SystemConstant.LOCAL;
                fileStoreUtil.localUpload(pdfDirName, new FileInputStream(pdfFileTemp), pdfFileMd5, LocalCatalogEnum.LOCAL_PDF);
            }

            PdfDto pdfDto = PdfUtil.addPdfPage(pdfFileTemp);

            JSONObject attachmentPath = JSON.parseObject(examDetail.getAttachmentPath());
            attachmentPath = Objects.isNull(attachmentPath) ? new JSONObject() : attachmentPath;
            JSONArray jsonArray = (JSONArray) attachmentPath.get(SystemConstant.PATH);
            jsonArray = Objects.isNull(jsonArray) ? new JSONArray() : jsonArray;
            JSONObject object = new JSONObject();
            object.put("printType", ClassifyEnum.PACKAGE.name());
            object.put("pdfMd5", pdfFileMd5);
            object.put(SystemConstant.PDF_PATH, pdfDirName);
            object.put(SystemConstant.TYPE, ossStr);
            object.put(SystemConstant.UPLOAD_TYPE, UploadFileEnum.PDF);
            jsonArray.add(object);
            attachmentPath.put(SystemConstant.PATH, jsonArray);
            examDetail.setAttachmentPath(attachmentPath.toJSONString());
            for (int i = 0; i < printCount; i++) {
                pdfList.add(new PdfDto(pdfFileTemp.getPath(), PageSizeEnum.A4, pdfDto.getPageCount(), 2));
            }
        } catch (Exception e) {
            log.error(SystemConstant.LOG_ERROR, e);
            if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        }
    }

    /**
     * 上传pdf文件
     *
     * @param pdfFile
     * @return
     */
    @Override
    public JSONObject uploadPdfFile(File pdfFile, ClassifyEnum classifyEnum) {
        try {
            boolean oss = dictionaryConfig.sysDomain().isOss();
            StringJoiner pdfStringJoiner = new StringJoiner("");
            if (!oss && Objects.nonNull(dictionaryConfig.fssPrivateDomain()) && !StringUtils.isBlank(dictionaryConfig.fssPrivateDomain().getConfig()) && !dictionaryConfig.fssPrivateDomain().getConfig().startsWith(SystemConstant.START_PARENT)) {
                pdfStringJoiner.add(dictionaryConfig.fssPrivateDomain().getConfig()).add(File.separator);
            }
            pdfStringJoiner = SystemConstant.getDirName(pdfStringJoiner, UploadFileEnum.PDF, true);
            pdfStringJoiner.add(SystemConstant.getNanoId()).add(SystemConstant.PDF_PREFIX);

            String pdfDirName = pdfStringJoiner.toString();
            pdfDirName = SystemConstant.getPdfFilePath(pdfDirName, pdfFile);

            JSONObject object = new JSONObject();
            object.put("printType", classifyEnum);
            object.put("pdfMd5", DigestUtils.md5Hex(new FileInputStream(pdfFile)));
            object.put(SystemConstant.PATH, pdfDirName);
            if (!oss) {
                object.put(SystemConstant.TYPE, SystemConstant.LOCAL);
            } else {
                object.put(SystemConstant.TYPE, SystemConstant.OSS);
            }
            object.put(SystemConstant.UPLOAD_TYPE, UploadFileEnum.PDF);
            return object;
        } catch (Exception e) {
            log.error(SystemConstant.LOG_ERROR, e);
            if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        }
        return null;
    }
}
